Package: dst
Type: Package
Title: Using the Theory of Belief Functions
Encoding: UTF-8
Version: 1.8.0
Date: 2024-08-24
Authors@R: c(person(given = "Peiyuan",
                      family = "Zhu",
                      role = c("aut","cre"),
                      email = "garyzhubc@gmail.com"),
             person(given = "Claude",
                      family = "Boivin",
                      role = c("aut"),
                      email = "webapp.cb@gmail.com"))
Description: Using the Theory of Belief Functions for evidence calculus. Basic probability assignments, or mass functions, can be defined on the subsets of a set of possible values and combined. A mass function can be extended to a larger frame. Marginalization, i.e. reduction to a smaller frame can also be done. These features can be combined to analyze small belief networks and take into account situations where information cannot be satisfactorily described by probability distributions.
License: GPL (>= 2)
LazyData: true
BugReports: https://github.com/RAPLER/dst-1/issues
RoxygenNote: 7.3.2
Depends: R (>= 3.5.0)
Imports: dplyr, ggplot2, tidyr, Matrix, methods, parallel, rlang, utils
Suggests: igraph, knitr, rmarkdown, tidyverse, testthat
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2024-09-03 03:57:14 UTC; garyzhubc
Author: Peiyuan Zhu [aut, cre],
  Claude Boivin [aut]
Maintainer: Peiyuan Zhu <garyzhubc@gmail.com>
Repository: CRAN
Date/Publication: 2024-09-03 04:20:06 UTC
Built: R 4.6.0; ; 2026-04-29 15:55:21 UTC; unix
