# Generated by roxygen2: do not edit by hand

export(DoSSnames)
export(addTobca)
export(bca)
export(bcaNorm)
export(bcaPrint)
export(bcaPrintLarge)
export(bcaRel)
export(bcaTrunc)
export(belplau)
export(belplauEval)
export(belplauH)
export(belplauHLogsumexp)
export(belplauHQQ)
export(belplauLogsumexp)
export(belplauPlot)
export(commonality)
export(decode)
export(dotprod)
export(doubles)
export(dsrwon)
export(dsrwonLogsumexp)
export(elim)
export(encode)
export(extFrame)
export(extmin)
export(inters)
export(intersBySSName)
export(logsum)
export(mFromMarginal)
export(mFromQQ)
export(mFromQQRecursive)
export(marrayToMatrix)
export(matrixToMarray)
export(mobiusInvHQQ)
export(nameCols)
export(nameCols_prod)
export(nameRows)
export(nzdsr)
export(nzdsrLogsumexp)
export(peeling)
export(plautrans)
export(productSpace)
export(reduction)
export(shape)
export(tabresul)
export(ttmatrix)
export(ttmatrixFromMarginal)
export(ttmatrixFromQQ)
export(ttmatrixPartition)
import(methods)
importClassesFrom(Matrix,RsparseMatrix)
importFrom(dplyr,"%>%")
importFrom(dplyr,group_by)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,summarise)
importFrom(ggplot2,aes)
importFrom(ggplot2,cut_width)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,ylab)
importFrom(rlang,":=")
importFrom(rlang,sym)
importFrom(stats,ftable)
importFrom(stats,quantile)
importFrom(stats,reorder)
importFrom(tidyr,unnest_wider)
importFrom(utils,object.size)
