# dst
Using Dempster-Shafer Theory of Evidence, also called "Theory of Belief Functions". Basic probability assignments, or mass functions, can be defined on the subsets of a set of possible values. Two mass functions on a variable A can be combined using Dempster's rule of combination. Relations between two variables A and B can be characterized by a mass functions defined on their product space A x B. A mass function on a variable A can be extended to the frame A x B. Dempster's rule of combination can be applied to product space. Marginalization, namely reduction to a smaller frame can also be done. These features can be combined to analyze small belief networks described by an hypergraph and take into account situations where information cannot be satisfactorily described by probability distributions. An algorithm, the peeling, is provided to compute belief functions in a hypergraph.
# Installation
Install from CRAN:
install.package("dst")
# Examples
See vignettes.
